@@ -23,15 +23,15 @@ import AST
 import CST
 import Data.List (intercalate, nub)
 import Data.Maybe (fromMaybe)
-import Encoding (Encoding (ASCII), ToASCII, withEncoding)
-import Lining (LineFormat (MULTILINE), ToSingleLine, withLineFormat)
+import Encoding
+import Lining
 import Locator (locatedExpression)
 import Margin (WithMargin, defaultMargin, withMargin)
 import Matcher
 import Misc
 import Render (Render (render))
 import Replacer (replaceProgram)
-import Rewriter (Rewritten)
+import Rewriter (Rewritten, Rewrittens)
 import Sugar (SugarType (SWEET), ToSalty, withSugarType)
 import Text.Printf (printf)
 import qualified Yaml as Y
@@ -51,7 +51,7 @@ data LatexContext = LatexContext
   }
 
 defaultLatexContext :: LatexContext
-defaultLatexContext = LatexContext SWEET MULTILINE defaultMargin False False defaultMeetPopularity defaultMeetLength ExGlobal Nothing Nothing Nothing
+defaultLatexContext = LatexContext SWEET SINGLELINE defaultMargin False False defaultMeetPopularity defaultMeetLength ExGlobal Nothing Nothing Nothing
 
 defaultMeetPopularity :: Int
 defaultMeetPopularity = 50
@@ -149,31 +149,32 @@ body printed toLatex =
         printed
     )
 
-ending :: LatexContext -> String
-ending ctx = printf "{.}\n\\end{%s}" (phiquation ctx)
+ending :: Bool -> LatexContext -> String
+ending True ctx = printf " \\leadsto\n  \\leadsto \\dots\n\\end{%s}" (phiquation ctx)
+ending False ctx = printf "{.}\n\\end{%s}" (phiquation ctx)
 
 compressedRewrittens :: [Rewritten] -> LatexContext -> [Rewritten]
 compressedRewrittens rewrittens ctx@LatexContext{..} =
   let (progs, rules) = unzip rewrittens
    in if _compress then zip (meetInPrograms progs ctx) rules else rewrittens
 
-rewrittensToLatex :: [Rewritten] -> LatexContext -> IO String
-rewrittensToLatex rewrittens ctx@LatexContext{_focus = ExGlobal} =
+rewrittensToLatex :: Rewrittens -> LatexContext -> IO String
+rewrittensToLatex (rewrittens, exceeded) ctx@LatexContext{_focus = ExGlobal} =
   pure
     ( concat
         [ preamble ctx
         , body (compressedRewrittens rewrittens ctx) (\prog -> renderToLatex (programToCST prog) ctx)
-        , ending ctx
+        , ending exceeded ctx
         ]
     )
-rewrittensToLatex rewrittens ctx@LatexContext{..} = do
+rewrittensToLatex (rewrittens, exceeded) ctx@LatexContext{..} = do
   let (progs, rules) = unzip (compressedRewrittens rewrittens ctx)
   exprs <- mapM (locatedExpression _focus) progs
   pure
     ( concat
         [ preamble ctx
         , body (zip exprs rules) (\expr -> renderToLatex (expressionToCST expr) ctx)
-        , ending ctx
+        , ending exceeded ctx
         ]
     )
 
@@ -182,7 +183,7 @@ programToLaTeX prog ctx =
   concat
     [ preamble ctx
     , renderToLatex (programToCST prog) ctx
-    , ending ctx
+    , ending False ctx
     ]
 
 expressionToLaTeX :: Expression -> LatexContext -> String
@@ -190,7 +191,7 @@ expressionToLaTeX ex ctx =
   concat
     [ preamble ctx
     , renderToLatex (expressionToCST ex) ctx
-    , ending ctx
+    , ending False ctx
     ]
 
 piped :: String -> String
@@ -212,10 +213,14 @@ instance ToLaTeX EXPRESSION where
   toLaTeX EX_DISPATCH{..} = EX_DISPATCH (toLaTeX expr) (toLaTeX attr)
   toLaTeX EX_PHI_MEET{..} = EX_PHI_MEET prefix idx (toLaTeX expr)
   toLaTeX EX_PHI_AGAIN{..} = EX_PHI_AGAIN prefix idx (toLaTeX expr)
+  toLaTeX EX_META{..} = EX_META (META NO_EXCL E' (rest meta))
   toLaTeX expr = expr
 
 instance ToLaTeX ATTRIBUTE where
   toLaTeX AT_LABEL{..} = AT_LABEL (piped (toLaTeX label))
+  toLaTeX AT_META{..} = AT_META (META NO_EXCL TAU' (rest meta))
+  toLaTeX AT_LAMBDA{} = AT_LAMBDA LAMBDA'
+  toLaTeX AT_REST{} = AT_REST DOTS'
   toLaTeX attr = attr
 
 instance ToLaTeX APP_BINDING where
@@ -223,10 +228,12 @@ instance ToLaTeX APP_BINDING where
 
 instance ToLaTeX BINDING where
   toLaTeX BI_PAIR{..} = BI_PAIR (toLaTeX pair) (toLaTeX bindings) tab
+  toLaTeX BI_META{..} = BI_META (META NO_EXCL B' (rest meta)) (toLaTeX bindings) tab
   toLaTeX bd = bd
 
 instance ToLaTeX BINDINGS where
   toLaTeX BDS_PAIR{..} = BDS_PAIR eol tab (toLaTeX pair) (toLaTeX bindings)
+  toLaTeX BDS_META{..} = BDS_META eol tab (META NO_EXCL B' (rest meta)) (toLaTeX bindings)
   toLaTeX bds = bds
 
 instance ToLaTeX PAIR where
@@ -236,6 +243,10 @@ instance ToLaTeX PAIR where
   toLaTeX PA_VOID{..} = PA_VOID (toLaTeX attr) arrow void
   toLaTeX PA_TAU{..} = PA_TAU (toLaTeX attr) arrow (toLaTeX expr)
   toLaTeX PA_FORMATION{..} = PA_FORMATION (toLaTeX attr) (map toLaTeX voids) arrow (toLaTeX expr)
+  toLaTeX PA_META_DELTA{..} = toLaTeX (PA_META_DELTA' meta)
+  toLaTeX PA_META_DELTA'{..} = PA_META_DELTA' (META NO_EXCL D' (rest meta))
+  toLaTeX PA_META_LAMBDA{..} = toLaTeX (PA_META_LAMBDA' meta)
+  toLaTeX PA_META_LAMBDA'{..} = PA_META_LAMBDA' (META NO_EXCL F (rest meta))
   toLaTeX pair = pair
 
 instance ToLaTeX APP_ARG where
@@ -257,24 +268,74 @@ instance ToLaTeX String where
         '_' -> "\\char95{}" <> escapeUnprintedChars rest
         _ -> ch : escapeUnprintedChars rest
 
--- @todo #114:30min Implement LaTeX conversion for rules.
---  Convert Rule data structure to LaTeX inference rule format.
---  Each rule should be formatted as a LaTeX inference rule with
---  pattern, result, and optional conditions.
---  Tests must be added for LaTeX conversion logic.
-explainRule :: Y.Rule -> String
-explainRule rule = "\\rule{" ++ fromMaybe "unnamed" (Y.name rule) ++ "}"
+instance ToLaTeX SET where
+  toLaTeX ST_BINDING{..} = ST_BINDING (toLaTeX binding)
+  toLaTeX ST_ATTRIBUTES{..} = ST_ATTRIBUTES (map toLaTeX attrs)
 
--- @todo #114:30min Create LaTeX document wrapper.
---  Generate proper LaTeX document with tabular format for rules.
---  Each rule should be   in its own tabular environment.
---  Include tests for document structure generation.
+instance ToLaTeX NUMBER where
+  toLaTeX ORDINAL{..} = ORDINAL (toLaTeX attr)
+  toLaTeX LENGTH{..} = LENGTH (toLaTeX binding)
+  toLaTeX literal@LITERAL{} = literal
+
+instance ToLaTeX COMPARABLE where
+  toLaTeX CMP_EXPR{..} = CMP_EXPR (toLaTeX expr)
+  toLaTeX CMP_ATTR{..} = CMP_ATTR (toLaTeX attr)
+  toLaTeX CMP_NUM{..} = CMP_NUM (toLaTeX num)
+
+instance ToLaTeX CONDITION where
+  toLaTeX CO_BELONGS{..} = CO_BELONGS (toLaTeX attr) belongs (toLaTeX set)
+  toLaTeX CO_LOGIC{..} = CO_LOGIC (map toLaTeX conditions) operator
+  toLaTeX CO_NF{..} = CO_NF (toLaTeX expr)
+  toLaTeX CO_NOT{..} = CO_NOT (toLaTeX condition)
+  toLaTeX CO_COMPARE{..} = CO_COMPARE (toLaTeX left) equal (toLaTeX right)
+  toLaTeX CO_MATCHES{..} = CO_MATCHES regex (toLaTeX expr)
+  toLaTeX CO_PART_OF{..} = CO_PART_OF (toLaTeX expr) (toLaTeX binding)
+  toLaTeX CO_EMPTY = CO_EMPTY
+
+instance ToLaTeX EXTRA_ARG where
+  toLaTeX ARG_ATTR{..} = ARG_ATTR (toLaTeX attr)
+  toLaTeX ARG_EXPR{..} = ARG_EXPR (toLaTeX expr)
+  toLaTeX ARG_BINDING{..} = ARG_BINDING (toLaTeX binding)
+  toLaTeX bts@ARG_BYTES{} = bts
+
+instance ToLaTeX EXTRA where
+  toLaTeX EXTRA{..} = EXTRA (toLaTeX meta) func (map toLaTeX args)
+
+explainRule :: Y.Rule -> String
+explainRule rule =
+  intercalate
+    "\n  "
+    [ "\\trrule{" ++ fromMaybe "unknown" (Y.name rule) ++ "}"
+    , braced (renderToLatex (expressionToCST (Y.pattern rule)) defaultLatexContext)
+    , braced (renderToLatex (expressionToCST (Y.result rule)) defaultLatexContext)
+    , conditionToLatex (joinedConditions (Y.when rule) (Y.having rule))
+    , extraArgumentsToLatex (Y.where_ rule)
+    ]
+  where
+    conditionToLatex :: Maybe Y.Condition -> String
+    conditionToLatex Nothing = "{ }"
+    conditionToLatex (Just cond) = case conditionToCST cond of
+      CO_EMPTY -> "{ }"
+      cond' -> braced ("if " <> renderToLatex cond' defaultLatexContext)
+    extraArgumentsToLatex :: Maybe [Y.Extra] -> String
+    extraArgumentsToLatex Nothing = "{ }"
+    extraArgumentsToLatex (Just extras) =
+      let extras' = map ((`renderToLatex` defaultLatexContext) . extraToCST) extras
+       in braced ("where " <> intercalate "; " extras')
+    braced :: String -> String
+    braced = printf "{ %s }"
+    -- Join two maybe conditions into single one using Y.And if at least one is just.
+    joinedConditions :: Maybe Y.Condition -> Maybe Y.Condition -> Maybe Y.Condition
+    joinedConditions Nothing Nothing = Nothing
+    joinedConditions first@(Just _) Nothing = first
+    joinedConditions Nothing second@(Just _) = second
+    joinedConditions (Just first) (Just second) = Just (Y.And [first, second])
+
 explainRules :: [Y.Rule] -> String
-explainRules rules' =
-  unlines
-    [ "\\documentclass{article}"
-    , "\\usepackage{amsmath}"
-    , "\\begin{document}"
+explainRules rules =
+  intercalate
+    "\n"
+    [ "\\begin{tabular}{rl}"
+    , intercalate "\n" (map explainRule rules)
+    , "\\end{tabular}"
     ]
-    ++ unlines (map explainRule rules')
-    ++ "\\end{document}"
