@@ -9,6 +9,7 @@ module Render where
 
 import CST
 import Data.List (intercalate)
+import Text.Printf (printf)
 
 class Render a where
   render :: a -> String
@@ -76,6 +77,7 @@ instance Render XI where
 
 instance Render LAMBDA where
   render LAMBDA = "λ"
+  render LAMBDA' = "\\lambda"
 
 instance Render GLOBAL where
   render Φ = "Φ"
@@ -92,23 +94,35 @@ instance Render EOL where
   render EOL = "\n"
   render NO_EOL = ""
 
+instance Render DOTS where
+  render DOTS = "..."
+  render DOTS' = "\\dots"
+
 instance Render BYTES where
   render BT_EMPTY = "--"
   render (BT_ONE bte) = render bte <> "-"
   render (BT_MANY bts) = intercalate "-" bts
   render (BT_META mt) = render mt
 
+instance Render EXCLAMATION where
+  render EXCL = "!"
+  render NO_EXCL = ""
+
+instance Render META_HEAD where
+  render E = "𝑒"
+  render E' = "e"
+  render A = "a"
+  render TAU = "𝜏"
+  render TAU' = "\\tau"
+  render B = "𝐵"
+  render B' = "B"
+  render D = "δ"
+  render D' = "d"
+  render TAIL = "t"
+  render F = "F"
+
 instance Render META where
-  render MT_EXPRESSION{..} = '𝑒' : rest
-  render MT_EXPRESSION'{..} = "!e" <> rest
-  render MT_ATTRIBUTE{..} = '𝜏' : rest
-  render MT_ATTRIBUTE'{..} = "!a" <> rest
-  render MT_BINDING{..} = '𝐵' : rest
-  render MT_BINDING'{..} = "!B" <> rest
-  render MT_BYTES{..} = 'δ' : rest
-  render MT_BYTES'{..} = "!d" <> rest
-  render MT_TAIL{..} = "!t" <> rest
-  render MT_FUNCTION{..} = "!F" <> rest
+  render META{..} = render excl <> render hd <> render rest
 
 instance Render ALPHA where
   render ALPHA = "α"
@@ -185,3 +199,53 @@ instance Render ATTRIBUTE where
   render AT_LAMBDA{..} = render lambda
   render AT_DELTA{..} = render delta
   render AT_META{..} = render meta
+  render AT_REST{..} = render dots
+
+instance Render BELONGING where
+  render IN = "\\in"
+  render NOT_IN = "\\notin"
+
+instance Render SET where
+  render ST_BINDING{..} = render binding
+  render ST_ATTRIBUTES{..} = printf "[ %s ]" (intercalate ", " (map render attrs))
+
+instance Render LOGIC_OPERATOR where
+  render AND = "and"
+  render OR = "or"
+
+instance Render NUMBER where
+  render ORDINAL{..} = renderFunc "ordinal" attr
+  render LENGTH{..} = renderFunc "length" binding
+  render LITERAL{..} = show num
+
+instance Render COMPARABLE where
+  render CMP_ATTR{..} = render attr
+  render CMP_EXPR{..} = render expr
+  render CMP_NUM{..} = render num
+
+instance Render EQUAL where
+  render EQUAL = "="
+  render NOT_EQUAL = "\\not="
+
+instance Render CONDITION where
+  render CO_BELONGS{..} = render attr <> render SPACE <> render belongs <> render SPACE <> render set
+  render CO_LOGIC{conditions = [cond]} = render cond
+  render CO_LOGIC{..} = printf "\\( %s \\)" (intercalate (printf " \\) %s \\( " (render operator)) (map render conditions))
+  render CO_NF{..} = renderFunc "NF" expr
+  render CO_NOT{..} = renderFunc "not" condition
+  render CO_COMPARE{..} = render left <> render SPACE <> render equal <> render SPACE <> render right
+  render CO_MATCHES{..} = printf "matches( %s, %s )" regex (render expr)
+  render CO_PART_OF{..} = printf "part-of( %s, %s )" (render expr) (render binding)
+  render CO_EMPTY = ""
+
+renderFunc :: Render a => String -> a -> String
+renderFunc func renderable = printf "%s( %s )" func (render renderable)
+
+instance Render EXTRA_ARG where
+  render ARG_ATTR{..} = render attr
+  render ARG_EXPR{..} = render expr
+  render ARG_BINDING{..} = render binding
+  render ARG_BYTES{..} = render bytes
+
+instance Render EXTRA where
+  render EXTRA{..} = render meta <> " <- " <> printf "%s( %s )" func (intercalate ", " (map render args))
