@@ -559,6 +559,19 @@ spec = do
               ]
           ]
 
+    it "shows exceeding of limits in latex" $
+      withStdin "{[[ x -> $.y, y -> $.x ]].x}" $
+        testCLISucceeded
+          ["rewrite", "--normalize", "--flat", "--sequence", "--output=latex", "--sweet", "--max-depth=1", "--max-cycles=1"]
+          [ unlines
+              [ "\\begin{phiquation}"
+              , "\\Big\\{[[ |x| -> |y|, |y| -> |x| ]].|x|\\Big\\} \\leadsto_{\\nameref{r:dot}}"
+              , "  \\leadsto \\Big\\{[[ |x| -> |y|, |y| -> |x| ]].|y|( ^ -> [[ |x| -> |y|, |y| -> |x| ]] )\\Big\\} \\leadsto"
+              , "  \\leadsto \\dots"
+              , "\\end{phiquation}"
+              ]
+          ]
+
     it "focuses expression in phi without sequence" $
       withStdin "{[[ ex -> [[ x -> [[ y -> ?, k -> [[ t -> 42]]  ]]( y -> [[ t -> 42 ]]) ]].i ]]}" $
         testCLISucceeded
@@ -806,17 +819,85 @@ spec = do
     it "explains single rule" $
       testCLISucceeded
         ["explain", "--rule=resources/copy.yaml"]
-        ["\\documentclass{article}", "\\usepackage{amsmath}", "\\begin{document}", "\\rule{COPY}", "\\end{document}"]
+        [ unlines
+            [ "\\begin{tabular}{rl}"
+            , "\\trrule{COPY}"
+            , "  { [[ B1, \\tau -> ?, B2 ]]( \\tau -> e1 ) }"
+            , "  { [[ B1, \\tau -> e3, B2 ]] }"
+            , "  { if NF( e1 ) }"
+            , "  { where e2 <- scope( e1 ); e3 <- contextualize( e1, e2 ) }"
+            , "\\end{tabular}"
+            ]
+        ]
 
     it "explains multiple rules" $
       testCLISucceeded
         ["explain", "--rule=resources/copy.yaml", "--rule=resources/alpha.yaml"]
-        ["\\documentclass{article}", "\\rule{COPY}", "\\rule{ALPHA}"]
+        ["\\begin{tabular}{rl}", "\\trrule{COPY}", "\\trrule{ALPHA}"]
 
     it "explains normalization rules" $
       testCLISucceeded
         ["explain", "--normalize"]
-        ["\\documentclass{article}", "\\begin{document}", "\\end{document}"]
+        [ unlines
+            [ "\\begin{tabular}{rl}"
+            , "\\trrule{ALPHA}"
+            , "  { [[ B1, \\tau1 -> ?, B2 ]]( \\tau2 -> e ) }"
+            , "  { [[ B1, \\tau1 -> ?, B2 ]]( \\tau1 -> e ) }"
+            , "  { if ordinal( \\tau2 ) = length( B1 ) }"
+            , "  { }"
+            , "\\trrule{COPY}"
+            , "  { [[ B1, \\tau -> ?, B2 ]]( \\tau -> e1 ) }"
+            , "  { [[ B1, \\tau -> e3, B2 ]] }"
+            , "  { if NF( e1 ) }"
+            , "  { where e2 <- scope( e1 ); e3 <- contextualize( e1, e2 ) }"
+            , "\\trrule{DC}"
+            , "  { T( \\tau -> e ) }"
+            , "  { T }"
+            , "  { }"
+            , "  { }"
+            , "\\trrule{DD}"
+            , "  { T.\\tau }"
+            , "  { T }"
+            , "  { }"
+            , "  { }"
+            , "\\trrule{DOT}"
+            , "  { [[ B1, \\tau -> e1, B2 ]].\\tau }"
+            , "  { e2( ^ -> [[ B1, \\tau -> e1, B2 ]] ) }"
+            , "  { if NF( e1 ) }"
+            , "  { where e2 <- contextualize( e1, [[ B1, \\tau -> e1, B2 ]] ) }"
+            , "\\trrule{MISS}"
+            , "  { [[ B ]]( \\tau -> e ) }"
+            , "  { T }"
+            , "  { if \\( \\tau \\notin B \\) and \\( \\tau \\notin [ ~0, ~1, \\dots ] \\) }"
+            , "  { }"
+            , "\\trrule{NULL}"
+            , "  { [[ B1, \\tau -> ?, B2 ]].\\tau }"
+            , "  { T }"
+            , "  { }"
+            , "  { }"
+            , "\\trrule{OVER}"
+            , "  { [[ B1, \\tau -> e1, B2 ]]( \\tau -> e2 ) }"
+            , "  { T }"
+            , "  { if \\tau \\not= ^ }"
+            , "  { }"
+            , "\\trrule{PHI}"
+            , "  { [[ B ]].\\tau }"
+            , "  { [[ B ]].@.\\tau }"
+            , "  { if \\( @ \\in B \\) and \\( \\tau \\notin B \\) }"
+            , "  { }"
+            , "\\trrule{STAY}"
+            , "  { [[ B1, ^ -> e1, B2 ]]( ^ -> e2 ) }"
+            , "  { [[ B1, ^ -> e1, B2 ]] }"
+            , "  { }"
+            , "  { }"
+            , "\\trrule{STOP}"
+            , "  { [[ B ]].\\tau }"
+            , "  { T }"
+            , "  { if \\( \\tau \\notin B \\) and \\( @ \\notin B \\) and \\( \\lambda \\notin B \\) }"
+            , "  { }"
+            , "\\end{tabular}"
+            ]
+        ]
 
     it "fails with no rules specified" $
       testCLIFailed
@@ -837,8 +918,8 @@ spec = do
             testCLISucceeded ["explain", "--normalize", printf "--target=%s" path] []
             content <- readFile path
             _ <- evaluate (length content)
-            content `shouldContain` "\\documentclass{article}"
-            content `shouldContain` "\\begin{document}"
+            content `shouldContain` "\\begin{tabular}{rl}"
+            content `shouldContain` "\\end{tabular}"
         )
 
   describe "merge" $ do
